import { CONSTANT_PLUGIN_TYPES } from './app.constant';
import { DATA_BASE_TYPES } from '@constants/constant';

BI.provider('dec.connection.provider.datebase', function () {
    this.resolves = {};

    function starRocksResolve(url: string) {
        // 处理starRocks数据连接常规模式
        let result = url.match(/^jdbc:mysql:\/\/([0-9a-zA-Z_\\.-]+):([0-9a-zA-Z_\\.-]+)\/([0-9a-zA-Z_\\.-]+)\.([^]+)(.*)/i);
        if (result) {
            return {
                host: result[1],
                port: result[2] === 'port' ? '' : result[2],
                catalog: result[3],
                databaseName: result[4],
                urlInfo: result[0],
            };
        } else {
            // 兼容老数据库里面没有catalog的情况
            result = url.match(/^jdbc:mysql:\/\/([0-9a-zA-Z_\\.-]+):([0-9a-zA-Z_\\.-]+)\/([^]+)(.*)/i);
            if (result) {
                return {
                    host: result[1],
                    port: result[2] === 'port' ? '' : result[2],
                    catalog: '',
                    databaseName: result[3],
                    urlInfo: result[0],
                };
            }
        }
        // 处理starRocks数据连接负载均衡模式
        let loadBalance = url.match(/^jdbc:mysql:loadbalance:\/\/[^/]+\/([^/]+)\.([^/]+)/i);
        if (loadBalance) {
            return {
                host: '',
                port: '',
                catalog: loadBalance[1],
                databaseName: loadBalance[2],
                urlInfo: loadBalance[0],
            }
        } else {
            // 兼容老数据库里面没有catalog的情况
            loadBalance = url.match(/^jdbc:mysql:loadbalance:\/\/[^/]+\/([^/]+)([^/]+)/i);
            if (loadBalance) {
                return {
                    host: '',
                    port: '',
                    catalog: '',
                    databaseName: loadBalance[1],
                    urlInfo: loadBalance[0],
                }
            }
        }
        return {
            host: '',
            port: '',
            databaseName: '',
            urlInfo: '',
        };
    }
    // 原service中resolveUrlInfo方法
    function jdbcResolve(url: string) {
        if (BI.isNull(url)) return {};

        const oracleUlr = url.match(/^jdbc:(oracle):(thin:([0-9a-zA-Z/]*)?@|thin:([0-9a-zA-Z/]*)?@\/\/|\/\/|)([0-9a-zA-Z_\\.-]+)(:([0-9|port]+))?(:|\/)([^]+)(.*)/i);
        if (oracleUlr) {
            return {
                host: oracleUlr[5],
                port: oracleUlr[7] === 'port' ? '' : oracleUlr[7],
                databaseName: oracleUlr[9],
                urlInfo: oracleUlr[10],
            };
        }

        const greenplumUrl = url.match(/^jdbc:(pivotal:greenplum):(thin:([0-9a-zA-Z/]*)?@\/\/|\/\/|)([0-9a-zA-Z_\\.-]+)(:([0-9|port]+))?(\/|;DatabaseName=)?([^]+)(.*)/i);
        if (greenplumUrl) {
            return {
                host: greenplumUrl[4],
                port: greenplumUrl[6] === 'port' ? '' : greenplumUrl[6],
                databaseName: greenplumUrl[8],
                urlInfo: greenplumUrl[9],
            };
        }
        const result = url.match(/^jdbc:(mysql|sqlserver|db2|dm|impala|kylin|phoenix|derby|gbase|gbasedbt-sqli|informix-sqli|h2|postgresql|hive2|vertica|kingbase|presto|redshift|postgresql|clickhouse|trino|sybase:Tds):(thin:([0-9a-zA-Z/]*)?@|thin:([0-9a-zA-Z/]*)?@\/\/|\/\/|)([0-9a-zA-Z_\\.-]+)(:([0-9|port]+))?(\/|;DatabaseName=)?([^]+)?(.*)/i);
        if (result) {
            return {
                host: result[5],
                port: result[7] === 'port' ? '' : result[7],
                databaseName: result[9] || '',
                urlInfo: result[10],
            };
        }

        // 处理SAP HANA数据连接url
        const sapHanaUrl = url.match(/^jdbc:(sap):(thin:([0-9a-zA-Z/]*)?@|thin:([0-9a-zA-Z/]*)?@\/\/|\/\/|)([0-9a-zA-Z_\\.-]+)(:([0-9|port]+))?(\?databaseName=)?([^&]+)([^]+)?(.*)/i);
        if (sapHanaUrl) {
            return {
                host: sapHanaUrl[5],
                port: sapHanaUrl[7] === 'port' ? '' : sapHanaUrl[7],
                databaseName: sapHanaUrl[9] || '',
                urlInfo: sapHanaUrl[10],
            };
        }

        // 处理oracle的RAC方式
        if (/^jdbc:oracle:thin:([0-9a-zA-Z/]*)?@\(DESCRIPTION/i.test(url)) {
            const host = url.match(/\(HOST\s*=\s*([0-9a-zA-Z_\\.-]+)\)/i);
            const port = url.match(/\(PORT\s*=\s*([0-9]+)\)/i);
            const databaseName = url.match(/\(SERVICE_NAME\s*=\s*([\s0-9a-zA-Z_\\.]+)\)/i);

            return {
                host: host ? host[1] : '',
                port: port && port[1] !== 'port' ? port[1] : '',
                databaseName: databaseName ? databaseName[1] : '',
                urlInfo: '',
            };
        }
        return {
            host: '',
            port: '',
            databaseName: '',
            urlInfo: '',
        };
    }

    function coverBaseDatabase(config) {
        const baseDataBase = DATA_BASE_TYPES.find(item => item.text === config.text);
        if (BI.isNotNull(baseDataBase)) {
            // 覆盖基础配置
            Object.assign(baseDataBase, config);

            return true;
        }

        return false;
    }

    function filterPluginDataTypeByPriority() {
       const originTypes = [...BI.Constants.getConstant(CONSTANT_PLUGIN_TYPES)];
       const sortDataTypes = BI.sortBy(originTypes, (index, value: any) => {
          return value.priority || 0;
       })
        return BI.uniqWith(sortDataTypes, (current, other) => {
            return current.text == other.text;
        });
    }

    this.registerDatabaseType = (config: any) => {
        if (coverBaseDatabase(config)) return;

        BI.config(CONSTANT_PLUGIN_TYPES, connections => BI.concat(connections, config));
    };

    this.registerJdbcDatabase = (config: any, resolve?: Function) => {
        config = {
            ...config,
            type: 'jdbc',
        };
        BI.isFunction(resolve) && (this.resolves[config.databaseType] = resolve);

        if (coverBaseDatabase(config)) return;

        BI.config(CONSTANT_PLUGIN_TYPES, connections => BI.concat(connections, config));
    };

    this.$get = () => BI.inherit(BI.OB, {
        getJdbcResolveByType: (type: string) => {
            // starRocks特殊处理
            // todo: 后面有专门的迭代系统处理，这里先临时解决下bug
            if (type === "starrocks"){
                return starRocksResolve
            }
            return this.resolves[type] || jdbcResolve
        },
        customDatabaseType: BI.Constants.getConstant(CONSTANT_PLUGIN_TYPES),
    });
});
